"""
format things for LaTeX
"""
from si_prefix import si_format as si

def listish(x):
    """
    Does it smell like a list?
    >>> listish(1)
    False
    >>> listish((1,2,3))
    True
    >>> listish([1,2,3])
    True
    """
    result = True
    try:
        len(x)
    except TypeError:
        result = False
    
    return result

def latex_float(f, units="", fmt="{0:.3g}"):
    r"""
    convert scientific formats to LaTeXish.
    if units are specific, add them in roman font
    
    >>> latex_float(3.2)
    '3.2'
    >>> latex_float(3.2e-12)
    '3.2 \\times 10^{-12}'
    >>> latex_float(3.2e-12,units='km')
    '3.2 \\times 10^{-12}\\,{\\rm km}'
    """
    
    if f == 0:
        f = 0  # fix annoying `-0` results.
    float_str = fmt.format(f)
    if "e" in float_str:
        base, exponent = float_str.split("e")
        result = r"{0} \times 10^{{{1}}}".format(base, int(exponent))
    else:
        result = float_str

    if units:
       result = result + r"\,{\rm " + units + "}"

    return result


def latex_vec(v, units="", fmt="{0:.3g}"):
    r"""
    v could be a vpython vec/vector object, or an iterable
    >>> latex_vec([1,2,3])
    '\\langle1,2,3\\rangle'
    >>> import vpython as vp
    >>> latex_vec(vp.vec(1,2,3))
    '\\langle1,2,3\\rangle'
    >>> latex_vec(vp.vec(1,2,3)*1e-10)
    '\\langle1 \\times 10^{-10},2 \\times 10^{-10},3 \\times 10^{-10}\\rangle'
    >>> latex_vec(vp.vec(1,2,3)*1e-10,units="m")
    '\\langle1 \\times 10^{-10},2 \\times 10^{-10},3 \\times 10^{-10}\\rangle\\,{\\rm m}'
    """
    if not listish(v):
        v = (v.x, v.y, v.z) # it must be a vpython vector, convert to tuple
    v = f"<{latex_float(v[0], fmt=fmt)},{latex_float(v[1], fmt=fmt)},{latex_float(v[2], fmt=fmt)}>"
    v = v.replace('<',r'\langle')
    v = v.replace('>',r'\rangle')
    if units:
        v = v + r"\,{\rm " + units + "}"
    return v

def eunits(val, units):
    r"""
    v could be a vpython vec/vector object, or an iterable
    >>> eunits(0.003,"A")
    '3.00\\,mA'
    """
    si_val = si(val,precision=4)
    num,pfx=si_val.split(' ')
    ip,fp = num.split('.')
    len_fp = max(3 - len(ip),0)
    if len_fp:
        fp = fp[:len_fp]
        num = '.'.join([ip,fp])
    else:
        num = ip
    pfx = pfx.replace(chr(181),r'\mu ') # take care of micro units
    si_val = r'\,'.join([num,pfx])
    return si_val + units


if __name__=='__main__':
    import doctest
    doctest.testmod()



"""
The `genlatex` module is for creating LaTeX documents from Jijna2 templates.
"""

version = '0.25'

from .genLatex import main
from .formatters import latex_float, latex_vec, eunits
from .texData import TeXData




from .texData import TeXData

def ConvertOldStyleDataGenerator( templateValues ):
    # old style data generator
    data = templateValues['data']
    answers = templateValues['answers']
    templateFile = templateValues['templateFile']
    quizFilenameTemplate = templateValues['quizFilenameTemplate']
    quizSolnFilenameTemplate = templateValues['quizSolnFilenameTemplate']
    
    promptData = {'data': data }
    promptDataSet = TeXData('prompt', quizFilenameTemplate, promptData)
    answerData = {'data': data, 'ans': answers}
    answerDataSet = TeXData('answers', quizSolnFilenameTemplate, answerData)

    return templateFile, [promptDataSet, answerDataSet]



#!/usr/bin/env python

import argparse
import inspect
import jinja2
import os
import math
import random
import importlib.util
import sys
from inspect import signature

from .convertOldStyle import ConvertOldStyleDataGenerator
from .texData import TeXData

def checkDataSet(dataSet, args):
    """
    Should we deal with this data set?
    """
    result = False
    if args.include:
        if dataSet.name in args.include:
            result = True
    elif args.exclude:
        if dataSet.name not in args.exclude:
            result = True
    else:
        result = True
    return result

def createTeXs(latex_jinja_env,
               path, seeds, args,
               templateFile,
               dataSets, prefix=''):

    digits = math.ceil(math.log(max(seeds)+1,10)) # zfill digits

    for dataSet in dataSets:
        if checkDataSet(dataSet, args):
            fnameTemplate = os.path.sep.join([path, prefix + dataSet.fileNameTemplate])
            template = latex_jinja_env.get_template(templateFile)
            for i in range(dataSet.getSize()):
                cnt = str(seeds[i]).zfill(digits)
                dataDict = {}
                for k in dataSet.templateData:
                    if len(k) > 3 and k[:3] == 'def':
                        dataDict[k[3:]] = dataSet.templateData[k]
                    else:
                        dataDict[k] = dataSet.templateData[k][i]

                texPromptSource = template.render(**dataDict)
                fname = fnameTemplate.format(cnt)
                f = open(fname,'w')
                f.write(texPromptSource)
                f.close()

def main():
    parser = argparse.ArgumentParser( description="Generate LateX Files from a template and a python helper",
                                    epilog="and that's how you'd generate some LaTeX files.")

    parser.add_argument('file_path')
    parser.add_argument('--seed','-s', nargs='+',default=None, type=int, help='seed value(s)')
    parser.add_argument('--num','-n',action='store', default=1, type=int, help='number of outputs')
    parser.add_argument('--exclude', '-e', nargs='+', help='exclude these data keys')
    parser.add_argument('--include', '-i', nargs='+', help='include these data keys')
    parser.add_argument('--debug', '-d', action='store_true', help='debug mode on')
    parser.add_argument('--srcdir', '-r', help='force source dir to be this')
    parser.add_argument('--prefix', '-p', default='', help='prefix for filename')
    parser.add_argument('--var', '-v', default='', help='define a template variable')

    args = parser.parse_args()

    if args.debug:
        import pdb
        pdb.set_trace()

    def seedFunc(i, seeds=None):
        return seeds and (seeds[i:] and seeds[i:][0] or (seeds[-1]+(i-len(seeds)+1))) or i 
        
    seeds = [seedFunc(i, args.seed) for i in range(args.num)] # get seeds, len = args.num by constr.

    # For illustrative purposes.
    file_path = args.file_path
    path,fname = os.path.split(file_path)
    module_name, ext = os.path.splitext(fname)

    if args.srcdir:
        #
        # when running in a container, the user will map /work/src to their cwd. Need to fix this up
        # to find the module correctly in the mapped drive
        #

        path = (path and os.path.join(args.srcdir,path)) or args.srcdir
        file_path = os.path.join(path, fname)
    elif not path:
        path = '.'

    if ext != '.py':
        print("Only python files can be imported.")
        sys.exit(-1)

    spec = importlib.util.spec_from_file_location(module_name, file_path)
    module = importlib.util.module_from_spec(spec)
    sys.modules[module_name] = module
    spec.loader.exec_module(module)

    latex_jinja_env = jinja2.Environment(
        block_start_string = r'\BLOCK{',
        block_end_string = '}',
        variable_start_string = r'\VAR{',
        variable_end_string = '}',
        comment_start_string = r'\#{',
        comment_end_string = '}',
        line_statement_prefix = '%%',
        line_comment_prefix = '%#',
        trim_blocks = True,
        autoescape = False,
        loader = jinja2.FileSystemLoader(os.path.abspath(path))
    )

    sig = inspect.signature(module.getTemplateValues)
    if (len(sig.parameters) > 2) and (args.var):
        templateValues = module.getTemplateValues(args.num, seeds, args.var)
    else:
        templateValues = module.getTemplateValues(args.num, seeds)

    if 'answers' in templateValues:
        templateFile, dataSets = ConvertOldStyleDataGenerator(templateValues)
    else:
        templateFile = templateValues['templateFile']
        dataSets = templateValues['dataSets']
    
    createTeXs(latex_jinja_env, path, seeds, args, templateFile, dataSets, args.prefix)

if __name__ == '__main__':
    main()



class TeXData:
    """
    Class to keep a set of data for rendering a LaTeX template

    name: the name of the dataset
    fileNameTemplate: a template to generate the filename for the LaTeX file.  Needs a slot for the file number (a string)
    templateData: a dictionary with template data. Each key should have a list of dicts with data

    templateData = {'data':[{'a':3, 'b':2}, {'a':9, 'b':12}, ...], 'answers':[{'c':5}, {'c':21}, ...}]}
    """

    def __init__(self, name, fileNameTemplate, templateData):
        self.name = name
        self.fileNameTemplate = fileNameTemplate
        self.templateData = templateData

    def getSize(self):
        """
        take a stab at determining the size of the template data.
        assume each key has a list of the same length. It must!
        """
        keys = list(self.templateData.keys())
        result = 0
        if keys:
            result = len(self.templateData[keys[0]])
        return result




from setuptools import setup

with open('README.md') as f:
    long_description = f.read()

from genlatex import version

setup(name='genlatex',
      version=version,
      description='A simple template based doc generator',
      long_description=long_description,
      long_description_content_type='text/markdown',
      url='https://github.com/sspickle/genlatex',
      author='Steve Spicklemire',
      author_email='steve@spvi.com',
      license='MIT',
      packages=['genlatex'],
      install_requires=[
          'Jinja2',
          'si-prefix'
      ],
      setup_requires=[
          'Jinja2',
          'si-prefix'
      ],
      zip_safe=False,
      entry_points = {
          'console_scripts': ['genlatex=genlatex.genLatex:main'],
      },
)
